@@ -19,6 +19,8 @@ t_optimize_sampling_plan <- function(alpha, beta, p_alpha, p_beta, n_init, k_ini
 {
   # Define function to calculate Pa based on the non-central t-distribution
   Pa_tdist <- function(n, k, p) {
+    # enforce n >= 2 to have valid df = n - 1
+    n <- max(2, n)
     # Calculate the acceptance probability using non-central t-distribution
     non_central_t <- pt(k * sqrt(n), df = n - 1, ncp = -qnorm(p) * sqrt(n))
     return(1 - non_central_t)
@@ -42,7 +44,19 @@ t_optimize_sampling_plan <- function(alpha, beta, p_alpha, p_beta, n_init, k_ini
   
   # Solve the optimization problem
   result <- optim(init_params, objective_function)
-  return(result$par)  # Return optimized n_s and k_s
+  
+  # Handle cases where the optimization fails to find an optimal solution
+  if (result$convergence != 0) {
+    return(list(
+      success = FALSE,
+      code    = result$convergence,
+      message = result$message
+    ))
+  }
+  
+  # Optimal plan found!
+  return(list(success = TRUE, n = result$par[1], k = result$par[2]))
+  #return(result$par)  # Return optimized n_s and k_s
 }
 
 #' Variable Acceptance Sampling Plan
@@ -138,10 +152,32 @@ optVarPlan <- function(PRQ, CRQ, alpha = 0.05, beta = 0.10,
       # find n_s by approximate from n
       # n <- ceiling(n) * (1 + k^2/2)
       
-      # Get optimal n_s and k_s set n,k as init value for optimize function
-      optimal_params <- t_optimize_sampling_plan(alpha, beta, PRQ, CRQ, n, k) 
-      n = optimal_params[1]
-      k = optimal_params[2]
+      # init value of n, k for optimization process
+      n_init <- max(2, n)
+      k_init <- k
+      
+      # # Get optimal n_s and k_s set n,k as init value for optimize function
+      # optimal_params <- t_optimize_sampling_plan(alpha, beta, PRQ, CRQ, 
+      #                                            n_init, k_init) 
+      # # final plan (n,k) after optimized
+      # n = optimal_params[1]
+      # k = optimal_params[2]
+      
+      # Get optimal n_s and k_s by optimizing
+      opt <- t_optimize_sampling_plan(alpha, beta, PRQ, CRQ, n_init, k_init)
+      
+      # Check if optimization was successful
+      if (!opt$success) {
+        stop(
+          paste("Optimization failed:",
+                opt$message,
+                "Please check inputs: alpha/beta, PRQ/CRQ or try different initial values.")
+        )
+      }
+      
+      # If success, set final plan
+      n <- opt$n
+      k <- opt$k
     }
     
     sample_size <- n
@@ -152,67 +188,15 @@ optVarPlan <- function(PRQ, CRQ, alpha = 0.05, beta = 0.10,
     
     r_alpha <- 1 - accProb(objPlan, PRQ)
     r_beta <- accProb(objPlan, CRQ)
-    m <- NA  # Not applicable for normal
+    #m <- NA  # Not applicable for normal
   } else {
-    # nsim = 5000 
-    # grid_step = c(0.5, 0.02)
-    # p1 <- PRQ
-    # p2 <- CRQ
-    # # Get normal plan for initial estimates
-    # normal_plan <- optVarPlan(PRQ = p1, CRQ = p2, 
-    #                           spec_limit = spec_limit, 
-    #                           distribution = "normal",
-    #                           limit_type = limit_type)
-    # # Set search ranges if not provided
-    # m_range <- c(floor(0.5 * normal_plan$n), ceiling(2 * normal_plan$n))
-    # k_range <- c(max(0.5 * normal_plan$k, 0.1), 2 * normal_plan$k)
-    # 
-    # # Create parameter grid
-    # m_values <- seq(m_range[1], m_range[2], by = grid_step[1])
-    # k_values <- seq(k_range[1], k_range[2], by = grid_step[2])
-    # search_grid <- expand.grid(m = m_values, k = k_values)
-    # 
-    # # Risk calculation function
-    # calculate_risks <- function(m, k) {
-    #   planObj <- structure(list(m = m, k = k, spec_limit = spec_limit, theta = theta,
-    #                             distribution = distribution, limtype = limit_type), 
-    #                        class = "VarPlan")
-    #   PR <- 1 - accProb(planObj, p1)
-    #   CR <- accProb(planObj, p2)
-    #   return(c(CR = CR, PR = PR))
-    # }
-    # 
-    # # Evaluate all grid points
-    # risks <- mapply(calculate_risks, search_grid$m, search_grid$k)
-    # search_grid$CR <- risks[1, ]
-    # search_grid$PR <- risks[2, ]
-    # 
-    # # Filter valid plans
-    # valid_plans <- subset(search_grid, CR <= beta & PR <= alpha)
-    # valid_plans <- valid_plans[order(valid_plans$m, valid_plans$k), ]
-    # 
-    # # Not found any optimal plan
-    # if (nrow(valid_plans) == 0)   return(NULL) #stop("No valid plan found in search range")
-    # 
-    # # Return optimal plan
-    # # Now we found the optimal beta plan here
-    # opt_plan <- as.list(valid_plans[1, ])
-    # k = opt_plan$k
-    # m = opt_plan$m 
-    # r_beta = opt_plan$CR 
-    # r_alpha = opt_plan$PR 
-    
-    # Objective function: minimize m
-    objective_function <- function(params) {
-      return(params[1])  # Minimizing m
-    }
     
     # Constraint function: Ensure PR <= alpha and CR <= beta
     constraint_function <- function(params) {
-      m <- params[1]
+      n <- params[1]
       k <- params[2]
       
-      planObj <- structure(list(m = m, k = k, USL = USL, LSL = LSL,
+      planObj <- structure(list(n = n, k = k, USL = USL, LSL = LSL,
                                 theta = theta, theta_type = "known",
                                 distribution = distribution), 
                            class = "VarPlan")
@@ -221,53 +205,75 @@ optVarPlan <- function(PRQ, CRQ, alpha = 0.05, beta = 0.10,
       CR <- accProb(planObj, CRQ)
       
       # Apply large penalty if constraints are violated
-      penalty <- sum(pmax(PR - alpha, 0)) + sum(pmax(CR - beta, 0))
-      return(params[1] + 1000 * penalty) 
+      penalty <- (pmax(PR - alpha, 0))^2 + (pmax(CR - beta, 0))^2 # smoother penalty 
+      return(n + 1e4 * penalty)   # (objective + penalty combined)
     }
     
     # Get initial estimates using normal distribution
     normal_plan <- optVarPlan(PRQ = PRQ, CRQ = CRQ, distribution = "normal")
     
     # Define search ranges
-    m_range <- c(floor(0.5 * normal_plan$n), ceiling(2 * normal_plan$n))
+    n_range <- c(floor(0.5 * normal_plan$n), ceiling(2 * normal_plan$n))
     k_range <- c(max(0.5 * normal_plan$k, 0.1), 2 * normal_plan$k)
     
     # Initial guess 
     initial_guess <- c(normal_plan$n, normal_plan$k)
     
-    # Run optimization using optim (L-BFGS-B supports bounds)
+    #Run optimization using optim (L-BFGS-B supports bounds)
     result <- optim(
       par = initial_guess,
       fn = constraint_function,
       method = "L-BFGS-B",
-      lower = c(m_range[1], k_range[1]),
-      upper = c(m_range[2], k_range[2])
+      lower = c(n_range[1], k_range[1]),
+      upper = c(n_range[2], k_range[2])
     )
-    # Extract optimal values
-    m <- result$par[1]
-    k <- result$par[2]
     
-    if(theta_type == "unknown") {
+    if (result$convergence == 0) {
+      # full success
+      n_opt <- result$par[1]
+      k_opt <- result$par[2]
       
-      ## This R ratio from paper of Govindaraju and Kissling (2015)
-      R_ratio = (1 + 0.85*k^2)
-      ## This R ration from my simulation
-      #R_ratio = (1 + 0.5*k^2)
-      # By theoretically motivated adjustment 
-      #R_ratio <- (1 + k^2)/2
+    } else if (result$convergence == 51) {
+      # warning but solution likely usable
+      warning("Optimization returned a warning (code 51): ", result$message)
+      n_opt <- result$par[1]
+      k_opt <- result$par[2]
       
-      m <- ceiling(m)*R_ratio
+    } else if (result$convergence == 52) {
+      # error — check if par is still valid
+      if (all(is.finite(result$par))) {
+        warning("Optimization returned a warning (code 52): ", result$message)
+        n_opt <- result$par[1]
+        k_opt <- result$par[2]
+      } else {
+        stop("Optimization failed with convergence code 52: ", result$message)
+      }
+      
+    } else {
+      stop("Optimization failed with convergence code:", result$convergence)
     }
     
-    objPlan <- structure(list(m = m, k = k, USL = USL, LSL = LSL,
+    # Extract optimal values
+    # n <- result$par[1]
+    # k <- result$par[2]
+    n <- n_opt
+    k <- k_opt
+    
+    if(theta_type == "unknown") {
+      ## This R ratio from paper of Govindaraju and Kissling (2015)
+      R_ratio = (1 + 0.85*k^2)
+      n <- ceiling(n)*R_ratio
+    }
+    
+    objPlan <- structure(list(n = n, k = k, USL = USL, LSL = LSL,
                               theta = theta, theta_type = theta_type,
                               distribution = distribution), 
                          class = "VarPlan")
     
     r_alpha <- 1 - accProb(objPlan, PRQ)
     r_beta <- accProb(objPlan, CRQ)
-    sample_size <- m
-    n <- m  # Not applicable for beta
+    sample_size <- n
+    #n <- m  # Not applicable for beta
   }
   
   return(structure(
@@ -280,7 +286,7 @@ optVarPlan <- function(PRQ, CRQ, alpha = 0.05, beta = 0.10,
       #spec_limit = spec_limit, limtype = limit_type,
       sample_size = ceiling(sample_size), # round up the sample size for practical;
       n = n, # we keep the number before round up
-      m = m, # we keep the number before round up
+      #m = m, # we keep the number before round up
       k = k
     ), 
     class = "VarPlan"
@@ -296,6 +302,10 @@ plot.VarPlan <- function(x, pd = NULL, by = c("pd", "mean"), ...) {
     pd <- seq(max(x$PRQ * 0.5, 1e-10), min(x$CRQ * 1.5, 1), length.out = 100)
   }
   
+  if (is.null(x$sigma_type)) {
+    stop("sigma_type must be provided to plot a VarPlan")
+  }
+  
   if (by == "pd") {
     pa <- sapply(pd, function(p) accProb(x, p))
     plot(pd, pa, type = "l", col = "red", lwd = 2,
